/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.saml2.provider.service.registration.RelyingPartyRegistrationRepository;
import org.springframework.security.saml2.provider.service.web.authentication.Saml2AuthenticationRequestResolver;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import de.ozgcloud.antragsraum.security.AntragsraumLogoutSuccessHandler;
import de.ozgcloud.antragsraum.security.AntragsraumProperties;
import de.ozgcloud.antragsraum.security.BayernIdProperties;
import de.ozgcloud.antragsraum.security.BayernIdSaml2Extension;
import de.ozgcloud.antragsraum.security.InMemoryUserDetailService;
import de.ozgcloud.antragsraum.security.JwtTokenFilter;
import de.ozgcloud.antragsraum.security.SamlUrlAuthenticationSuccessHandler;

@ExtendWith(MockitoExtension.class)
class SecurityConfigurationTest {
	@Mock
	private BayernIdSaml2Extension bayernIdSaml2Extension;

	@Mock
	private BayernIdProperties properties;

	@Mock
	private InMemoryUserDetailService userDetailsService;

	@Mock
	private JwtTokenFilter tokenFilter;

	@Mock
	private AntragsraumProperties corsProperties;

	private SecurityConfiguration securityConfiguration;

	@BeforeEach
	void setUp() {
		securityConfiguration = new SecurityConfiguration(bayernIdSaml2Extension, properties, userDetailsService, tokenFilter, corsProperties);
	}

	@Nested
	class TestHttpSecurity {
		private HttpSecurity httpSecurity;

		@BeforeEach
		void init() throws Exception {
			httpSecurity = mock(HttpSecurity.class);
			when(httpSecurity.authorizeHttpRequests(any())).thenReturn(httpSecurity);
			when(httpSecurity.saml2Login(any())).thenReturn(httpSecurity);
			when(httpSecurity.saml2Logout(any())).thenReturn(httpSecurity);
			when(httpSecurity.addFilterBefore(any(), any())).thenReturn(httpSecurity);
			when(httpSecurity.csrf(any())).thenReturn(httpSecurity);
		}

		@Test
		void shouldSetupFilterChainAuthorize() throws Exception {
			securityConfiguration.filterChain(httpSecurity);

			verify(httpSecurity).authorizeHttpRequests(any());
		}

		@Test
		void shouldSetupFilterChainSaml2Login() throws Exception {
			securityConfiguration.filterChain(httpSecurity);

			verify(httpSecurity).saml2Login(any());
		}

		@Test
		void shouldSetupFilterChainSaml2Logout() throws Exception {
			securityConfiguration.filterChain(httpSecurity);

			verify(httpSecurity).saml2Logout(any());
		}
	}

	@Nested
	class TestAuthenticationRequestResolver {
		@Mock
		private RelyingPartyRegistrationRepository registrations;

		@Test
		void shouldCreateSaml2AuthenticationRequestResolver() {
			var authResolver = securityConfiguration.authenticationRequestResolver(registrations);

			assertThat(authResolver).isInstanceOf(Saml2AuthenticationRequestResolver.class);
		}
	}

	@Nested
	class TestSamlFilterChain {
		@Mock
		HttpSecurity security;

		@BeforeEach
		void setUp() throws Exception {
			when(security.authorizeHttpRequests(any())).thenReturn(security);
			when(security.addFilterBefore(any(), any())).thenReturn(security);
			when(security.saml2Login(any())).thenReturn(security);
			when(security.saml2Logout(any())).thenReturn(security);
		}

		@Test
		void shouldSetSamlLogin() throws Exception {
			securityConfiguration.filterChain(security);

			verify(security).saml2Login(any());
		}

		@Test
		void shouldSetSamlLogout() throws Exception {
			securityConfiguration.filterChain(security);

			verify(security).saml2Logout(any());
		}
	}

	@Nested
	class TestLogoutHandler {
		@Test
		void shouldCreateLogoutHandler() {
			when(corsProperties.getLogoutSuccessUrl()).thenReturn("/");

			var handler = securityConfiguration.getLogoutSuccessHandler();

			assertThat(handler).isInstanceOf(AntragsraumLogoutSuccessHandler.class);
		}
	}

	@Nested
	class TestSuccessHandler {
		@Test
		void shouldCreateSuccessHandler() {
			var handler = securityConfiguration.getSamlUrlAuthenticationSuccessHandler();

			assertThat(handler).isInstanceOf(SamlUrlAuthenticationSuccessHandler.class);
		}
	}

	@Nested
	class TestCorsConfigurer {
		@Test
		void shouldCreateCorsConfigurer() {
			var corsConfigurer = securityConfiguration.corsConfigurer();

			assertThat(corsConfigurer).isInstanceOf(WebMvcConfigurer.class);
		}
	}
}