/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Date;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;

import de.ozgcloud.antragsraum.command.CommandReference;
import de.ozgcloud.antragsraum.command.CommandTestFactory;
import de.ozgcloud.antragsraum.common.AddressNotFoundException;
import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.events.NachrichtEventTestFactory;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcFindRueckfragenResponse;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcGetRueckfrageResponse;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcRueckfrage;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcRueckfrageAnswer;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcRueckfrageHead;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcSendRueckfrageAnswerResponse;
import de.ozgcloud.antragsraum.security.UserTestFactory;
import io.grpc.StatusRuntimeException;

@ExtendWith(MockitoExtension.class)
public class NachrichtenRemoteServiceTest {
	public static final String ADDRESS = "test";

	@Spy
	@InjectMocks
	private NachrichtenRemoteService nachrichtenRemoteService;

	@Mock
	private NachrichtenRestClient nachrichtenRestClient;

	@Mock
	private NachrichtMapper nachrichtMapper;

	@Mock
	private AntragraumProxyMapper antragraumProxyMapper;

	@BeforeEach
	void init() {
		var authenticated = mock(UsernamePasswordAuthenticationToken.class);
		when(authenticated.getPrincipal()).thenReturn(UserTestFactory.create());
		var context = mock(SecurityContext.class);
		when(context.getAuthentication()).thenReturn(authenticated);
		SecurityContextHolder.setContext(context);
	}

	@Nested
	class TestNachrichtenAvailable {
		private final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.createNachrichtEvent();

		@BeforeEach
		void init() {
			var reply = new AntragraumproxyGrpcFindRueckfragenResponse();
			var rueckfrageHeader = new AntragraumproxyGrpcRueckfrageHead();
			rueckfrageHeader.setAnsweredAt(new Date().toString());
			rueckfrageHeader.setSentAt(new Date().toString());
			reply.setRueckfrageHead(List.of(rueckfrageHeader));
			when(nachrichtenRestClient.getFindRueckfragen(anyString(), anyString(), anyString())).thenReturn(reply);
		}

		@Test
		void shouldLoadNachrichtHeaders() {
			var nachrichtHeaders = nachrichtenRemoteService.findRueckfrageHeads(nachrichtEvent);

			assertThat(nachrichtHeaders).isNotEmpty();
		}

		@Test
		void shouldSetSamlToken() {
			nachrichtenRemoteService.findRueckfrageHeads(nachrichtEvent);

			verify(nachrichtenRestClient).getFindRueckfragen(anyString(), anyString(), anyString());
		}
	}

	@Nested
	class TestNoNachrichtenAvailable {
		@BeforeEach
		void init() {
			doThrow(StatusRuntimeException.class).when(nachrichtenRestClient).getFindRueckfragen(anyString(), anyString(), anyString());
		}

		@Test
		void shouldHandleStatusRuntimeException() {
			var nachrichtHeaders = nachrichtenRemoteService.findRueckfrageHeads(NachrichtEventTestFactory.createNachrichtEvent());

			assertThat(nachrichtHeaders).isEmpty();
		}
	}

	@Nested
	class TestNoTargetAddressAvailable {
		@BeforeEach
		void init() {
			doThrow(AddressNotFoundException.class).when(nachrichtenRestClient)
			  .getFindRueckfragen(anyString(), anyString(), anyString());
		}

		@Test
		void shouldThrowAddressNotFoundException() {
			var nachrichtEvent = NachrichtEventTestFactory.createNachrichtEvent();

			assertThatExceptionOfType(AddressNotFoundException.class).isThrownBy(() -> nachrichtenRemoteService.findRueckfrageHeads(nachrichtEvent));
		}
	}

	@Nested
	class TestGetNachricht {
		private final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.createNachrichtEvent();

		@BeforeEach
		void init() {
			when(nachrichtMapper.fromRestRueckfrage(any(), anyString())).thenReturn(NachrichtTestFactory.createNachricht());
			var reply = new AntragraumproxyGrpcGetRueckfrageResponse();
			reply.setRueckfrage(new AntragraumproxyGrpcRueckfrage());
			when(nachrichtenRestClient.getGetRueckfrage(anyString(), anyString(), anyString())).thenReturn(reply);
		}

		@Test
		void shouldLoadNachricht() {
			var nachricht = nachrichtenRemoteService.getRueckfrage(AntragraumproxyGrpcRueckfrageTestFactory.RUECKFRAGE_ID, nachrichtEvent);

			assertThat(nachricht).isNotEmpty();
		}

		@Test
		void shouldSetSamlToken() {
			nachrichtenRemoteService.getRueckfrage(AntragraumproxyGrpcRueckfrageTestFactory.RUECKFRAGE_ID, nachrichtEvent);

			verify(nachrichtenRestClient).getGetRueckfrage(anyString(), anyString(), anyString());
		}
	}

	@Nested
	class TestSendAnswer {
		private final ReplyNachricht reply = ReplyNachrichtTestFactory.create();

		@BeforeEach
		void init() {
			when(antragraumProxyMapper.toRestProxyAntwort(any())).thenReturn(new AntragraumproxyGrpcRueckfrageAnswer());
			var response = new AntragraumproxyGrpcSendRueckfrageAnswerResponse();
			response.setCommandId(CommandTestFactory.ID);
			when(nachrichtenRestClient.answerRueckfrage(any(AntragraumproxyGrpcRueckfrageAnswer.class),
			  anyString(), anyString())).thenReturn(response);
		}

		@Test
		void shouldSendAnswer() {
			var result = nachrichtenRemoteService.sendAnswer(reply, ADDRESS);

			assertThat(result).isInstanceOf(CommandReference.class);
		}

		@Test
		void shouldReturnCommandReference() {
			var result = nachrichtenRemoteService.sendAnswer(reply, ADDRESS);

			assertThat(result.id()).isEqualTo(CommandTestFactory.ID);
			assertThat(result.nachrichtEventId()).isEqualTo(ReplyNachrichtTestFactory.NACHRICHT_EVENT_ID);
		}

		@Test
		void shouldSetSamlResponse() {
			nachrichtenRemoteService.sendAnswer(reply, ADDRESS);

			verify(nachrichtenRestClient).answerRueckfrage(any(AntragraumproxyGrpcRueckfrageAnswer.class), anyString(), anyString());
		}
	}
}
